/*
 * Copyright (c) 2013, ZheJiang Uniview Technologies Co., Ltd. All rights reserved.
 * <http://www.uniview.com/>
 *------------------------------------------------------------------------------
 * Product     : IMOS
 * Module Name :
 * Date Created: 2013-11-5
 * Creator     : y00486
 * Description :
 *
 *------------------------------------------------------------------------------
 * Modification History
 * DATE        NAME             DESCRIPTION
 *
 *------------------------------------------------------------------------------
 */
package com.uniview.airimos.android_ts_sdk_sample;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.text.TextUtils;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.uniview.airimos.IMOSPlayer;
import com.uniview.airimos.listener.OnDragReplayListener;
import com.uniview.airimos.listener.OnGainBitRateListener;
import com.uniview.airimos.listener.OnGainLostPacketRateListener;
import com.uniview.airimos.listener.OnGetAlarmListener;
import com.uniview.airimos.listener.OnGetReplayPosListener;
import com.uniview.airimos.listener.OnPushAlarmListener;
import com.uniview.airimos.listener.OnQueryReplayListener;
import com.uniview.airimos.listener.OnQueryResourceListener;
import com.uniview.airimos.listener.OnStartLiveListener;
import com.uniview.airimos.listener.OnStartReplayListener;
import com.uniview.airimos.listener.OnStreamChangeListener;
import com.uniview.airimos.manager.InfoManager;
import com.uniview.airimos.manager.ServiceManager;
import com.uniview.airimos.obj.AlarmInfo;
import com.uniview.airimos.obj.LiveOrVodType;
import com.uniview.airimos.obj.QueryCondition;
import com.uniview.airimos.obj.RecordInfo;
import com.uniview.airimos.obj.ResourceInfo;
import com.uniview.airimos.parameter.PushAlarmParam;
import com.uniview.airimos.parameter.QueryReplayParam;
import com.uniview.airimos.parameter.QueryResourceParam;
import com.uniview.airimos.parameter.StartLiveParam;
import com.uniview.airimos.parameter.StartReplayParam;
import com.uniview.airimos.parameter.StreamParam;
import com.uniview.airimos.protocol.CameraInfo;
import com.uniview.airimos.service.KeepaliveService;
import com.uniview.airimos.service.KeepaliveService.KeepaliveBinder;
import com.uniview.airimos.service.KeepaliveService.OnKeepaliveListener;
import com.uniview.airimos.util.NetWorkConst;
import com.uniview.airimos.util.StreamUtil;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class LiveReplayActivity extends Activity implements OnClickListener, OnKeepaliveListener {
    private static final String TAG = "LiveReplayActivity";
    private IMOSPlayer mPlayer;
    private SurfaceView mSurfaceView;
    private Button mBtnStartLive;
    private Button mBtnStopLive;
    private Button mBtnStream;

    private Button mBtnStartReplay;
    private Button mBtnStopReplay;
    private Button mBtnDragReplay;
    private Button mBtnReplayPos;

    private Button mBtnSnatch;
    private Button mBtnRecord;
    private Button mBtnStopRecord;
    private Button mBtnMute;
    private Button mBtnQueryRes;
    private Button mBtnGetAlarm;
    private Button mBtnPushAlarm;
    private EditText mEditCamCode;
    private EditText mEditReplayBeginTime;
    private EditText mEditReplayEndTime;
    private KeepaliveService mService = null;
    private boolean mBound = false;
    private boolean mRequireLogout = false;

    private Button mPtzControl;

    private TextView mBitrate;
    private TextView mLostPacketRate;
    private EditText mGainBitratePeriod;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_live);
        //SurfaceView用于渲染
        mSurfaceView = (SurfaceView) findViewById(R.id.surface_view);

        //监听SurfaceView的变化
        mSurfaceView.getHolder().addCallback(new surfaceCallback());


        mBtnStartLive = (Button) findViewById(R.id.btn_start_live);
        mBtnStopLive = (Button) findViewById(R.id.btn_stop_live);
        mBtnStream = (Button) findViewById(R.id.btn_stream);

        mBtnStartReplay = (Button) findViewById(R.id.btn_start_replay);
        mBtnStopReplay = (Button) findViewById(R.id.btn_stop_replay);
        mBtnDragReplay = (Button) findViewById(R.id.btn_drag_replay);
        mBtnReplayPos = (Button) findViewById(R.id.btn_replay_pos);

        mEditReplayBeginTime = (EditText) findViewById(R.id.edit_replay_begin_time);
        mEditReplayEndTime = (EditText) findViewById(R.id.edit_replay_end_time);
        mBtnSnatch = (Button) findViewById(R.id.btn_snatch);
        mPtzControl = (Button) findViewById(R.id.ptzControl);
        mBtnRecord = (Button) findViewById(R.id.btn_record);
        mBtnStopRecord = (Button) findViewById(R.id.btn_stop_record);
        mBtnMute = (Button) findViewById(R.id.btn_mute);
        mBtnGetAlarm = (Button) findViewById(R.id.btn_get_alarm);
        mBtnPushAlarm = (Button) findViewById(R.id.btn_push_alarm);

        mBtnQueryRes = (Button) findViewById(R.id.btn_query_res);
        mEditCamCode = (EditText) findViewById(R.id.edit_camera_code);

        mBitrate = (TextView) findViewById(R.id.tv_bitrate);
        mLostPacketRate = (TextView) findViewById(R.id.tv_lost_packet_rate);
        mGainBitratePeriod = (EditText) findViewById(R.id.et_gain_bitrate_period);

        //初始化一个Player对象
        mPlayer = new IMOSPlayer();
        mPlayer.setPlayerCb(new IMOSPlayer.IMOSPlayerCallback() {
            @Override
            public void onStartPlay() {
                // 起流开始，出现画面
                // 启动获取码率和丢包率 任务
                updatePacketInfo(mPlayer.getPlaySession(), mPlayer.getCameraCode(), mCurrentPlayType);
            }

            @Override
            public void showError(int errorCode) {
                // 播放出现错误，此处应给予相应提示和处理
//                Toast.makeText(MainActivity.this, "errorCode:" + errorCode + ";errorInfo:" + NetWorkConst.getErrorInfoByCode(errorCode), Toast.LENGTH_LONG).show();
            }

            @Override
            public void showBitrate(final double bitrate) {
                // 起流成功后，会回调实时码率值
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        mBitrate.setText(getString(R.string.bitrate, String.valueOf(bitrate)));
                    }
                });
            }
        });
        mPlayer.setDisplay(mSurfaceView.getHolder());
        mPlayer.setStreamProtocol(IMOSPlayer.STREAM_PROTOCOL_UDP);
    }

    @Override
    protected void onStart() {

        //启动保活服务。若不想用提供的保活服务类，可通过每十秒调用保活接口ServiceManager.keepalive实现保活
        if (!mBound) {
            Intent intent = new Intent(this, KeepaliveService.class);
            startService(intent);
            bindService(intent, mConnection, Context.BIND_AUTO_CREATE);
        }

        mBtnStartLive.setOnClickListener(this);
        mBtnStopLive.setOnClickListener(this);
        mBtnStream.setOnClickListener(this);
        mBtnStartReplay.setOnClickListener(this);
        mBtnStopReplay.setOnClickListener(this);
        mBtnDragReplay.setOnClickListener(this);
        mBtnReplayPos.setOnClickListener(this);
        mBtnSnatch.setOnClickListener(this);
        mBtnQueryRes.setOnClickListener(this);
        mPtzControl.setOnClickListener(this);
        mBtnRecord.setOnClickListener(this);
        mBtnStopRecord.setOnClickListener(this);
        mBtnMute.setOnClickListener(this);
        mBtnGetAlarm.setOnClickListener(this);
        mBtnPushAlarm.setOnClickListener(this);


        //模拟一个查询回放起始和结束的时间
        Date now = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String begin = formatter.format(now) + " 00:00:00";
        String end = formatter.format(now) + " 23:59:59";
        mEditReplayBeginTime.setText(begin);
        mEditReplayEndTime.setText(end);


        super.onStart();
    }

    @Override
    protected void onStop() {
        stopUpdatePacketInfo();

        //解除保活服务的绑定
        if (mBound) {
            unbindService(mConnection);
            mBound = false;

            if (mRequireLogout) {
                Intent serviceIntent = new Intent(LiveReplayActivity.this, KeepaliveService.class);
                stopService(serviceIntent);
            }
        }

        super.onStop();
    }

    @Override
    protected void onDestroy() {
        //销毁Player
        if (mPlayer != null) {
            mPlayer.destroy();
            mPlayer = null;
        }

        //退出登录，一般在用户手动退出时调用，这里只作演示
        ServiceManager.logout(null);
        super.onDestroy();
    }

    /**
     * 启动实况
     *
     * @param cameraCode 摄像机编码
     */
    public void startLive(final String cameraCode) {
        try {
            //启动实况的结果监听
            OnStartLiveListener listener = new OnStartLiveListener() {
                @Override
                public void onStartLiveResult(long errorCode, String errorDesc, String playSession) {
                    mPlayer.stopPlay();
                    mCurrentPlayType = LiveOrVodType.LIVE_TYPE;

                    CameraInfo cameraInfo = new CameraInfo();
                    cameraInfo.setCameraCode(cameraCode);
                    mPlayer.startPlay(InfoManager.getServer(), InfoManager.getMediaPort(), playSession, cameraInfo);
                }

            };

            //设置启动实况的参数
            StartLiveParam p = new StartLiveParam();
            p.setCameraCode(cameraCode);
            p.setUseSecondStream(true); //使用辅流
            p.setBitrate(32 * 8);   //64KB的码率
            p.setFramerate(12);     //25帧率
            p.setResolution(StreamParam.ResolutionType.CIF);     //CIF分辨率

            //启动实况
            ServiceManager.startLive(p, listener);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void stopLive() {
        stopUpdatePacketInfo();

        //停止实况，第二个参数是null表示不接收结果
        ServiceManager.stopLive(mPlayer.getPlaySession(), null);

        //停止Player播放解码
        mPlayer.stopPlay();
    }

    public void pushAlarm(String cameraCode) {

        PushAlarmParam param = new PushAlarmParam();
        param.setCameraCode(cameraCode);
        param.setAlarmType(401);
        param.setAlarmDesc("推送告警测试");

        try {
            OnPushAlarmListener listener = new OnPushAlarmListener() {
                @Override
                public void onPushAlarmResult(long errorCode, String errorDesc) {
                    if (0 != errorCode) {
                        Toast.makeText(LiveReplayActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(LiveReplayActivity.this, "告警推送成功", Toast.LENGTH_SHORT).show();
                    }
                }
            };
            //推送告警接口
            ServiceManager.pushAlarmMessage(param, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void getAlarm(int timeout) {
        try {
            OnGetAlarmListener listener = new OnGetAlarmListener() {
                @Override
                public void onGetAlarmResult(long errorCode, String errorDesc, List<AlarmInfo> alarmList) {
                    if (null == alarmList) {
                        return;
                    }
                    if (errorCode != 0) {
                        Toast.makeText(LiveReplayActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        int size = alarmList.size();
                        StringBuffer stringBuffer = new StringBuffer();
                        for (int i = 0; i < size; i++) {
                            AlarmInfo alarmInfo = alarmList.get(i);
                            stringBuffer.append(alarmInfo.getAlarmSrcCode() + ",");
                            stringBuffer.append(alarmInfo.getAlarmSrcName() + ",");
                            stringBuffer.append(alarmInfo.getAlarmType() + ",");
                            stringBuffer.append(alarmInfo.getAlarmDesc() + ",");
                            stringBuffer.append(alarmInfo.getAlarmTime() + "\n");
                        }

                        Log.i(TAG, "告警信息:" + stringBuffer.toString());
                    }
                }
            };

            //获取告警
            ServiceManager.getAlarmMessage(timeout, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }



    public void ipcQuery(String org){
        //查询资源参数
        QueryResourceParam params = new QueryResourceParam(org, "", new QueryCondition(0, 200, false));

        //查询资源结果监听
        OnQueryResourceListener listener = new OnQueryResourceListener() {
            @Override
            public void onQueryResourceResult(long errorCode, String errorDesc, List<ResourceInfo> resList) {
                if (null == resList) {
                    return;
                }
                StringBuffer strBuf = new StringBuffer();
                int size = resList.size();
                for (int i = 0; i < size; i++) {
                    strBuf.append(resList.get(i).getResCode() + ",");
                    strBuf.append(resList.get(i).getResName() + ",");
                    strBuf.append(resList.get(i).getResType() + ",");
                    strBuf.append(resList.get(i).getResSubType() + ",");
                    strBuf.append(resList.get(i).getIsOnline() + "\n");

                    //Toast.makeText(LiveReplayActivity.this,"size:"+size+"  "+strBuf.toString(),Toast.LENGTH_LONG).show();
                    Log.d(TAG+"xxxxx", strBuf.toString());

                    if (resList.get(i).getResType() == ResourceInfo.ResType.CAMERA && resList.get(i).getIsOnline()) {
                        mEditCamCode.setText(resList.get(i).getResCode());
                        break;//只要查到一台能用的就停止查询
                    }else if(resList.get(i).getResType() == ResourceInfo.ResType.ORG){
                        ipcQuery(resList.get(i).getResCode());
                    }
                }
            }
        };

        //查询资源接口调用
        ServiceManager.queryResource(params, listener);
    }

    @Override
    public void onClick(View v) {
        if (mBtnStartLive.equals(v)) {
            String cameraCode = mEditCamCode.getText().toString();

            startLive(cameraCode);
        } else if (mBtnStopLive.equals(v)) {
            stopLive();
        } else if (mBtnSnatch.equals(v)) {
            //抓拍图片，返回路径
            String path = mPlayer.snatch(null);
            if (null != path) {
                Toast.makeText(LiveReplayActivity.this, path, Toast.LENGTH_SHORT).show();
            }
        } else if (mBtnQueryRes.equals(v)) {
            ipcQuery("");
        } else if (mBtnStartReplay.equals(v)) {

            final String cameraCode = mEditCamCode.getText().toString();
            String beginTime = mEditReplayBeginTime.getText().toString();
            String endTime = mEditReplayEndTime.getText().toString();

            //查询回放记录参数
            QueryReplayParam p = new QueryReplayParam(cameraCode, beginTime, endTime, new QueryCondition(0, 100, true));

            //查询回放记录结果监听
            OnQueryReplayListener queryListener = new OnQueryReplayListener() {
                @Override
                public void onQueryReplayResult(long errorCode, String errorDesc, List<RecordInfo> recordList) {
                    if (recordList == null || recordList.size() <= 0) {
                        Log.d(TAG, "There is no record");
                        return;
                    }


                    //取第一条回放记录测试回放
                    RecordInfo firstRecord = recordList.get(0);

                    //启动回放的参数
                    StartReplayParam p = new StartReplayParam();
                    p.setCameraCode(cameraCode);
                    p.setRecodeInfo(firstRecord);
                    p.setBitrate(64 * 8);  //64KB码率
                    p.setFramerate(20);     //20帧率
                    p.setResolution(2);     //4CIF分辨率


                    OnStartReplayListener listener = new OnStartReplayListener() {
                        @Override
                        public void onStartReplayResult(long errorCode, String errorDesc, String playSession) {
                            //先停掉已有的播放
                            mPlayer.stopPlay();

                            mCurrentPlayType = LiveOrVodType.VOD_TYPE;

                            //启动播放解码
                            CameraInfo cameraInfo = new CameraInfo();
                            cameraInfo.setCameraCode(cameraCode);
                            mPlayer.startPlay(InfoManager.getServer(), InfoManager.getMediaPort(), playSession, cameraInfo);
                        }
                    };

                    //启动回放
                    ServiceManager.startReplay(p, listener);
                }
            };

            //先查询指定时间段内有的回放记录
            ServiceManager.queryReplay(p, queryListener);
        } else if (mBtnStopReplay.equals(v)) {
            //停止回放
            ServiceManager.stopReplay(mPlayer.getPlaySession(), null);

            //停止播放解码
            mPlayer.stopPlay();
        } else if (mPtzControl.equals(v)) {
            Intent intent = new Intent();
            intent.setClass(LiveReplayActivity.this, PtzActivity.class);
            startActivity(intent);
        } else if (mBtnDragReplay.equals(v)) {
            //拖动回放，"2016-01-10 12:00:00"为要播放的时间，具体能用的时间在queryReplay中返回
            ServiceManager.dragReplay(mPlayer.getPlaySession(), "2016-01-10 12:00:00", new OnDragReplayListener() {
                @Override
                public void onDragReplayResult(long errorCode, String errorDesc) {

                }
            });
        } else if (mBtnRecord.equals(v)) {
            String path = mPlayer.startRecord(null);

            if (null != path) {
                Toast.makeText(LiveReplayActivity.this, "录像开始：" + path, Toast.LENGTH_SHORT).show();
            }
        } else if (mBtnStopRecord.equals(v)) {
            mPlayer.stopRecord();
            Toast.makeText(LiveReplayActivity.this, "录像完成", Toast.LENGTH_SHORT).show();
        } else if (mBtnMute.equals(v)) {

            // false为开启声音，若要静音传true
            mPlayer.setMute(false);
        } else if (mBtnGetAlarm.equals(v)) {
            // 查询告警为阻塞调用，有告警会立即返回。如果需要一直查询，可以放在线程中连续调用
            getAlarm(10000);
        } else if (mBtnPushAlarm.equals(v)) {
            String cameraCode = mEditCamCode.getText().toString();
            pushAlarm(cameraCode);
        } else if (mBtnReplayPos.equals(v)) {//获取当前回放播放时间
            ServiceManager.getReplayPos(mPlayer.getPlaySession(), new OnGetReplayPosListener() {
                @Override
                public void onGetReplayResult(String replayPos, long errorCode, String errorDesc) {

                    if (0 == errorCode) {
                        Toast.makeText(LiveReplayActivity.this, "当前播放时间" + replayPos, Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(LiveReplayActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    }

                }
            });
        } else if (mBtnStream.equals(v)) {//切换清晰度
            StreamParam streamParam = StreamUtil.getStreamInfo(StreamUtil.STREAM_TYPE_C, LiveReplayActivity.this);//D1 清晰码流

//            StreamParam customStreamParam = new StreamParam();//配置自定清晰度参数
//            customStreamParam.setStreamType(StreamUtil.STREAM_TYPE_CS);//设置清晰度类型为自定义类型
//            customStreamParam.setResolution(StreamParam.ResolutionType.P720);//设置分辨率
//            customStreamParam.setBitRate(1024);//设置码率
//            customStreamParam.setFrameRate(25);//设置帧率
//
//            StreamUtil.saveCustomStreamInfo(LiveReplayActivity.this, customStreamParam);//保存自定义的清晰度参数
//
//            StreamParam streamParam1 = StreamUtil.getCustomStreamInfo(LiveReplayActivity.this);//获取自定义清晰度参数
//            StreamParam streamParam1 = StreamUtil.getStreamInfo(StreamUtil.STREAM_TYPE_CS, LiveReplayActivity.this);//获取自定义清晰度参数


            ServiceManager.changeStream(mPlayer.getPlaySession(), streamParam, new OnStreamChangeListener() {
                @Override
                public void onStreamChangeResult(StreamParam streamParam, long errorCode, String errorDesc) {
                    if (0 == errorCode) {
                        Toast.makeText(LiveReplayActivity.this, "清晰度切换成功！", Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(LiveReplayActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    }
                }
            });


        }

    }


    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName className, IBinder service) {
            KeepaliveBinder binder = (KeepaliveBinder) service;
            mService = binder.getService();
            mService.start(LiveReplayActivity.this);
            mBound = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName arg0) {
            mBound = false;
        }
    };

    /**
     * 保活失败响应，退出
     */
    public void onKeepaliveFailed() {

        mRequireLogout = true;
        Intent intent = new Intent();
        intent.setClass(LiveReplayActivity.this, LoginActivity.class);
        startActivity(intent);
        Toast.makeText(LiveReplayActivity.this, "保活失败，已退出", Toast.LENGTH_LONG).show();
        finish();
    }

    ;


    class surfaceCallback implements SurfaceHolder.Callback {

        public void surfaceCreated(SurfaceHolder holder) {
            Log.d(TAG, "===== surfaceCreated =====");
        }

        public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            Log.d(TAG, "===== surfaceChanged ===== width " + width + " " + " height " + height);
            if (mPlayer != null) {
                mPlayer.changeDisplaySize(width, height);
            }

        }

        @Override
        public void surfaceDestroyed(SurfaceHolder arg0) {
            Log.d(TAG, "===== surfaceDestroyed =====");
        }
    }

    /////////////////////////////合入获取丢包率功能////////////////////////////////

    /**
     * 当前播放的实况or回放
     */
    private int mCurrentPlayType = LiveOrVodType.LIVE_TYPE;

    /**
     * 当前窗格播放实况/回放的丢包率
     * 若开启多路窗格，可创建多个回调变量
     */
    private OnGainLostPacketRateListener onGainLostPacketRateListener = new OnGainLostPacketRateListener() {
        @Override
        public void onGainLostPacketRateResult(String lostPacketRate, long errorCode, String errorDes) {
            Log.d(TAG, "丢包率：lostPacketRate:" + lostPacketRate + ";errorCode;" + errorCode + ";errorDes:" + errorDes);
//            Toast.makeText(MainActivity.this,
//                    "丢包率：lostPacketRate:" + lostPacketRate + ";errorCode;" + errorCode + ";errorDes:" + errorDes, Toast.LENGTH_SHORT).show();
            if (errorCode == NetWorkConst.NO_ERROR) {
                double lostPacketRateValue = Double.parseDouble(lostPacketRate);
                mLostPacketRate.setText(getString(R.string.lost_packet_rate, String.valueOf(lostPacketRateValue / 100)));
            }

            /*
             * @TODO 这里有一点需要注意，回调回来的丢包率有可能会出现异常值，即lostPacketRate >= 1.0
             * 此时需要调用{@link ServiceManager.gainResetLostPacketRate}
             * 复位丢包率
             */
        }
    };

    private Timer updatePacketInfoTimer;

    private TimerTask updatePacketInfoTimerTask;

    /**
     * 更新丢包率和码率信息，当重新起流出现图片的时候需要调用
     *
     * @param playSession 播放Session
     * @param cameraCode  相机编码
     * @param playType    播放类型，参考{@link com.uniview.airimos.obj.LiveOrVodType}
     */
    private void updatePacketInfo(final String playSession, final String cameraCode, final int playType) {
        Log.d(TAG, "begin update packetInfo ...");
        // 先停止之前的获取码率任务
        stopUpdatePacketInfo();

        updatePacketInfoTimer = new Timer();
        updatePacketInfoTimerTask = new TimerTask() {
            @Override
            public void run() {
                Log.d(TAG, "gainLostPacketRate and gainBitRate ...");
                ServiceManager.gainLostPacketRate(playSession, playType, onGainLostPacketRateListener);
            }
        };

        int gainRatePeriod = 5000;// 默认5秒获取一次
        if (!TextUtils.isEmpty(mGainBitratePeriod.getText())) {
            gainRatePeriod = Integer.parseInt(mGainBitratePeriod.getText().toString());
        }
        Log.d(TAG, "gainRatePeriod " + gainRatePeriod);
        updatePacketInfoTimer.schedule(updatePacketInfoTimerTask, 500, gainRatePeriod);
    }

    /**
     * 停止获取丢包率和码率
     */
    private void stopUpdatePacketInfo() {
        Log.d(TAG, "stop update packetInfo ...");
        if (null != updatePacketInfoTimerTask) {
            updatePacketInfoTimerTask.cancel();
            updatePacketInfoTimerTask = null;
        }
        if (null != updatePacketInfoTimer) {
            updatePacketInfoTimer.cancel();
            updatePacketInfoTimer = null;
        }
    }
}
