package com.uniview.airimos.android_ts_sdk_sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.uniview.airimos.IMOSPlayer;
import com.uniview.airimos.listener.OnDelPresetListener;
import com.uniview.airimos.listener.OnLockPtzListener;
import com.uniview.airimos.listener.OnPtzCommandListener;
import com.uniview.airimos.listener.OnQueryPresetListener;
import com.uniview.airimos.listener.OnQueryResourceListener;
import com.uniview.airimos.listener.OnSetPresetListener;
import com.uniview.airimos.listener.OnStartLiveListener;
import com.uniview.airimos.listener.OnStopLiveListener;
import com.uniview.airimos.listener.OnUnLockPtzListener;
import com.uniview.airimos.listener.OnUsePresetListener;
import com.uniview.airimos.manager.InfoManager;
import com.uniview.airimos.manager.ServiceManager;
import com.uniview.airimos.obj.QueryCondition;
import com.uniview.airimos.obj.ResourceInfo;
import com.uniview.airimos.parameter.LockPtzParam;
import com.uniview.airimos.parameter.PtzCommandParam;
import com.uniview.airimos.parameter.QueryPresetParam;
import com.uniview.airimos.parameter.QueryResourceParam;
import com.uniview.airimos.parameter.SetPresetParam;
import com.uniview.airimos.parameter.StartLiveParam;
import com.uniview.airimos.parameter.StreamParam;
import com.uniview.airimos.parameter.UseAndDelPresetParam;
import com.uniview.airimos.protocol.CameraInfo;
import com.uniview.airimos.protocol.PresetInfo;
import com.uniview.airimos.service.KeepaliveService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 云台控制界面
 * <p>
 * Created by l02465 on 2016/4/12.
 */
public class PtzActivity extends Activity implements KeepaliveService.OnKeepaliveListener, View.OnClickListener, AdapterView.OnItemClickListener, AdapterView.OnItemLongClickListener {
    private static final String TAG = "PtzActivity";
    private SurfaceView mSurfaceView;
    private EditText mEditCamCode;
    private Button mBtnQueryRes;
    private Button mBtnStartPreview;
    private Button mBtnStopPreview;
    private Button mBtnUp;
    private Button mBtnDown;
    private Button mBtnLeft;
    private Button mBtnRight;
    private Button mBtnStopPtz;
    private Button mBtnLockPtz;
    private Button mBtnUnlockPtz;
    private Button mBtnAddPreset;
    private Button mBtnQueryPreset;
    private EditText mEditPresetValue;
    private EditText mEditPresetDesc;
    private ListView mListPreset;

    private IMOSPlayer mPlayer;
    private KeepaliveService mService = null;
    private boolean mBound = false;
    private boolean mRequireLogout = false;
    private String mCameraCode;
    private List<PresetInfo> mPresetInfos;
    private ArrayAdapter mAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_ptz);

        mSurfaceView = (SurfaceView) findViewById(R.id.surface_view);
        mSurfaceView.getHolder().addCallback(new MySurfaceCallback());

        mEditCamCode = (EditText) findViewById(R.id.cameraCode);
        mBtnQueryRes = (Button) findViewById(R.id.queryPtz);
        mBtnStartPreview = (Button) findViewById(R.id.startLive);
        mBtnStopPreview = (Button) findViewById(R.id.stopLive);
        mBtnUp = (Button) findViewById(R.id.upPtz);
        mBtnLeft = (Button) findViewById(R.id.left);
        mBtnRight = (Button) findViewById(R.id.right);
        mBtnStopPtz = (Button) findViewById(R.id.stopPtz);
        mBtnDown = (Button) findViewById(R.id.down);
        mBtnLockPtz = (Button) findViewById(R.id.lockPtz);
        mBtnUnlockPtz = (Button) findViewById(R.id.unlockPtz);
        mEditPresetValue = (EditText) findViewById(R.id.presetValue);
        mEditPresetDesc = (EditText) findViewById(R.id.presetDesc);
        mBtnAddPreset = (Button) findViewById(R.id.addPreset);
        mBtnQueryPreset = (Button) findViewById(R.id.queryPreset);
        mListPreset = (ListView) findViewById(R.id.preset_list);

        mBtnStartPreview.setEnabled(false);
        mBtnStopPreview.setEnabled(false);
        mBtnUp.setEnabled(false);
        mBtnLeft.setEnabled(false);
        mBtnRight.setEnabled(false);
        mBtnDown.setEnabled(false);
        mBtnStopPtz.setEnabled(false);
        mBtnLockPtz.setEnabled(false);
        mBtnUnlockPtz.setEnabled(false);
        mBtnAddPreset.setEnabled(false);
        mBtnQueryPreset.setEnabled(false);

        mBtnQueryRes.setOnClickListener(this);
        mBtnStartPreview.setOnClickListener(this);
        mBtnStopPreview.setOnClickListener(this);
        mBtnDown.setOnClickListener(this);
        mBtnUp.setOnClickListener(this);
        mBtnLeft.setOnClickListener(this);
        mBtnRight.setOnClickListener(this);
        mBtnStopPtz.setOnClickListener(this);
        mBtnLockPtz.setOnClickListener(this);
        mBtnUnlockPtz.setOnClickListener(this);
        mBtnAddPreset.setOnClickListener(this);
        mBtnQueryPreset.setOnClickListener(this);

        mListPreset.setOnItemClickListener(this);
        mListPreset.setOnItemLongClickListener(this);


        //初始化一个Player对象
        mPlayer = new IMOSPlayer();
        mPlayer.setDisplay(mSurfaceView.getHolder());

        mEditCamCode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                mCameraCode = s.toString();
                if (null != mCameraCode) {
                    mBtnStartPreview.setEnabled(true);
                    mBtnStopPreview.setEnabled(true);
                    mBtnUp.setEnabled(true);
                    mBtnLeft.setEnabled(true);
                    mBtnRight.setEnabled(true);
                    mBtnStopPtz.setEnabled(true);
                    mBtnDown.setEnabled(true);
                    mBtnLockPtz.setEnabled(true);
                    mBtnUnlockPtz.setEnabled(true);
                    mBtnAddPreset.setEnabled(true);
                    mBtnQueryPreset.setEnabled(true);
                }
            }
        });
    }

    /**
     * 查询摄像机资源
     */
    public void queryCameraRes() {
        try {
            OnQueryResourceListener listener = new OnQueryResourceListener() {
                @Override
                public void onQueryResourceResult(long errorCode, String errorDesc, List<ResourceInfo> resList) {
                    if (null == resList) {
                        return;
                    }
                    StringBuffer stringBuffer = new StringBuffer();
                    int size = resList.size();
                    for (int i = 0; i < size; i++) {
                        stringBuffer.append(resList.get(i).getResCode() + ",");
                        stringBuffer.append(resList.get(i).getResName() + ",");
                        stringBuffer.append(resList.get(i).getResType() + ",");
                        stringBuffer.append(resList.get(i).getResSubType() + ",");
                        stringBuffer.append(resList.get(i).getIsOnline() + "\n");

                        //找到第一个在线的摄像机
                        if (resList.get(i).getResType() == ResourceInfo.ResType.CAMERA && resList.get(i).getIsOnline()) {
                            mEditCamCode.setText(resList.get(i).getResCode());
                            mCameraCode = resList.get(i).getResCode().trim();
                        }

                        if (!("").equals(mEditCamCode.getText().toString())) {
                            mBtnStartPreview.setEnabled(true);
                            mBtnStopPreview.setEnabled(true);
                            mBtnUp.setEnabled(true);
                            mBtnLeft.setEnabled(true);
                            mBtnRight.setEnabled(true);
                            mBtnStopPtz.setEnabled(true);
                            mBtnDown.setEnabled(true);
                            mBtnLockPtz.setEnabled(true);
                            mBtnUnlockPtz.setEnabled(true);
                            mBtnAddPreset.setEnabled(true);
                            mBtnQueryPreset.setEnabled(true);
                        }
                    }
                    Log.d(TAG, stringBuffer.toString());
                }
            };

            //查询摄像机资源参数
            QueryResourceParam param = new QueryResourceParam("", "", new QueryCondition(0, 100, true));

            //调用查询资源接口
            ServiceManager.queryResource(param, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * 启动实况
     *
     * @param cameraCode 摄像机编码
     */
    public void startLive(final String cameraCode) {
        try {
            //启动实况的结果监听
            OnStartLiveListener listener = new OnStartLiveListener() {
                @Override
                public void onStartLiveResult(long errorCode, String errorDesc, String playSession) {
                    mPlayer.stopPlay();

                    CameraInfo cameraInfo = new CameraInfo();
                    cameraInfo.setCameraCode(cameraCode);
                    mPlayer.startPlay(InfoManager.getServer(), InfoManager.getMediaPort(), playSession, cameraInfo);
                }

            };

            //设置启动实况的参数
            StartLiveParam p = new StartLiveParam();
            p.setCameraCode(cameraCode);
            p.setUseSecondStream(true); //使用辅流
            p.setBitrate(32 * 8);   //64KB的码率
            p.setFramerate(12);     //25帧率
            p.setResolution(StreamParam.ResolutionType.CIF);     //CIF分辨率

            //启动实况
            ServiceManager.startLive(p, listener);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void stopLive() {
        try {
            OnStopLiveListener listener = new OnStopLiveListener() {
                @Override
                public void onStopLiveResult(long errorCode, String errorDesc) {
                    //errorCode为0表示成功
                    if (errorCode == 0) {
                        Toast.makeText(PtzActivity.this, "停止实况成功", Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    }
                }
            };
            //停止实况接口调用
            ServiceManager.stopLive(mPlayer.getPlaySession(), listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }

        //停止Player播放解码
        mPlayer.stopPlay();
    }

    /**
     * 云台控制
     */
    public void ptzCommand(String cameraCode, int directionCode) {

        //云台命令参数
        PtzCommandParam param = new PtzCommandParam();
        param.setCameraCode(cameraCode);
        param.setCmd(directionCode);
        param.setSpeed1(5);
        param.setSpeed2(5);

        try {
            OnPtzCommandListener listener = new OnPtzCommandListener() {
                @Override
                public void onPtzCommandResult(long errorCode, String errorDesc) {
                    if (errorCode != 0) {
                        Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    }
                }
            };
            //云台控制接口
            ServiceManager.ptzCommand(param, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * 云台锁定
     */
    public void lockPtz(String cameraCode) {

        LockPtzParam param = new LockPtzParam();
        param.setCameraCode(cameraCode);

        try {
            OnLockPtzListener listener = new OnLockPtzListener() {
                @Override
                public void onLockPtzResult(long errorCode, String errorDesc) {
                    if (errorCode != 0) {
                        Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(PtzActivity.this, "云台锁定成功", Toast.LENGTH_SHORT).show();
                    }
                }
            };
            //锁定云台接口调用
            ServiceManager.lockPtz(param, listener);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    /**
     * 云台解锁
     */
    public void unLockPtz(String cameraCode) {

        LockPtzParam param = new LockPtzParam();
        param.setCameraCode(cameraCode);

        try {
            OnUnLockPtzListener listener = new OnUnLockPtzListener() {
                @Override
                public void onUnLockPtzResult(long errorCode, String errorDesc) {
                    if (errorCode != 0) {
                        Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(PtzActivity.this, "云台解锁成功", Toast.LENGTH_SHORT).show();
                    }
                }
            };
            //解锁云台接口调用
            ServiceManager.unLockPtz(param, listener);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * 添加预置位
     */
    public void addPreset(String cameraCode) {
        if (("").equals(mEditPresetValue.getText().toString()) && ("").equals(mEditPresetDesc.getText().toString())) {
            Toast.makeText(PtzActivity.this, "请输入预置位值和描述信息", Toast.LENGTH_SHORT).show();
        } else {
            int presetValue = Integer.valueOf(mEditPresetValue.getText().toString());
            String presetDesc = mEditPresetDesc.getText().toString();

            SetPresetParam param = new SetPresetParam();
            param.setCameraCode(cameraCode);
            param.setPresetValue(presetValue);
            param.setPresetDesc(presetDesc);

            try {
                OnSetPresetListener listener = new OnSetPresetListener() {
                    @Override
                    public void setPresetResult(long errorCode, String errorDesc) {
                        if (errorCode != 0) {
                            Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                        } else {
                            Toast.makeText(PtzActivity.this, "添加预置位成功", Toast.LENGTH_SHORT).show();
                        }
                    }
                };
                //添加预置位接口
                ServiceManager.addPreset(param, listener);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }

    }

    /**
     * 查询预置位
     */
    public void queryPreset(String cameraCode) {
        try {
            OnQueryPresetListener listener = new OnQueryPresetListener() {
                @Override
                public void onQueryPresetResult(long errorCode, String errorDesc, List<PresetInfo> list) {
                    if (null == list) {
                        return;
                    }
                    mPresetInfos = list;
                    int size = list.size();
                    StringBuffer stringBuffer = new StringBuffer();
                    ArrayList<HashMap<Integer, String>> presets = new ArrayList<>();

                    for (int i = 0; i < size; i++) {
                        stringBuffer.append(list.get(i).getPresetValue() + "\n");
                        stringBuffer.append(list.get(i).getPresetDesc() + "\n");

                        HashMap<Integer, String> item = new HashMap<>();
                        item.put(list.get(i).getPresetValue(), list.get(i).getPresetDesc());
                        presets.add(item);

                    }
                    Log.d(TAG, stringBuffer.toString());

                    mAdapter = new ArrayAdapter(PtzActivity.this, android.R.layout.simple_list_item_1, presets);
                    mListPreset.setAdapter(mAdapter);
                }
            };

            QueryPresetParam param = new QueryPresetParam(cameraCode, new QueryCondition(0, 100, true));
            //调用预置位查询接口
            ServiceManager.queryPreset(param, listener);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * 使用预置位
     */
    public void usePreset(String cameraCode, int presetValue) {
        try {
            OnUsePresetListener listener = new OnUsePresetListener() {
                @Override
                public void usePresetResult(long errorCode, String errorDesc) {
                    if (errorCode != 0) {
                        Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    }
                }
            };

            UseAndDelPresetParam param = new UseAndDelPresetParam(cameraCode, presetValue);
            //使用预置位
            ServiceManager.usePreset(param, listener);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * 删除预置位
     */
    public void delPreset(String cameraCode, int presetValue) {
        try {
            OnDelPresetListener listener = new OnDelPresetListener() {
                @Override
                public void delPresetListener(long errorCode, String errorDesc) {
                    if (errorCode != 0) {
                        Toast.makeText(PtzActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(PtzActivity.this, "删除成功", Toast.LENGTH_SHORT).show();
                    }
                }
            };

            UseAndDelPresetParam param = new UseAndDelPresetParam(cameraCode, presetValue);
            //删除预置位
            ServiceManager.delPreset(param, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        //启动保活服务。若不想用提供的保活服务类，可通过每十秒调用保活接口ServiceManager.keepalive实现保活
        if (!mBound) {
            Intent intent = new Intent(this, KeepaliveService.class);
            startService(intent);
            bindService(intent, mConnection, Context.BIND_AUTO_CREATE);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        mListPreset.setAdapter(mAdapter);
    }

    @Override
    protected void onStop() {
        //解除保活服务的绑定
        if (mBound) {
            unbindService(mConnection);
            mBound = false;

            if (mRequireLogout) {
                Intent serviceIntent = new Intent(PtzActivity.this, KeepaliveService.class);
                stopService(serviceIntent);
            }
        }

        super.onStop();
    }

    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName className, IBinder service) {
            KeepaliveService.KeepaliveBinder binder = (KeepaliveService.KeepaliveBinder) service;
            mService = binder.getService();
            mService.start(PtzActivity.this);
            mBound = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName arg0) {
            mBound = false;
        }
    };

    /**
     * 保活失败响应，退出
     */
    public void onKeepaliveFailed() {

        mRequireLogout = true;
        Intent intent = new Intent();
        intent.setClass(PtzActivity.this, LoginActivity.class);
        startActivity(intent);
        Toast.makeText(PtzActivity.this, "保活失败，已退出", Toast.LENGTH_LONG).show();
        finish();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.queryPtz:
                queryCameraRes();
                break;
            case R.id.startLive:
                startLive(mCameraCode);
                break;
            case R.id.stopLive:
                stopLive();
                break;
            case R.id.down:
                ptzCommand(mCameraCode, PtzCommandParam.PTZ_CMD.TILTDOWN);
                break;
            case R.id.upPtz:
                ptzCommand(mCameraCode, PtzCommandParam.PTZ_CMD.TILTUP);
                break;
            case R.id.left:
                ptzCommand(mCameraCode, PtzCommandParam.PTZ_CMD.PANLEFT);
                break;
            case R.id.right:
                ptzCommand(mCameraCode, PtzCommandParam.PTZ_CMD.PANRIGHT);
                break;
            case R.id.stopPtz:
                ptzCommand(mCameraCode, PtzCommandParam.PTZ_CMD.ALLSTOP);
                break;
            case R.id.lockPtz:
                lockPtz(mCameraCode);
                break;
            case R.id.unlockPtz:
                unLockPtz(mCameraCode);
                break;
            case R.id.addPreset:
                addPreset(mCameraCode);
                mAdapter.notifyDataSetChanged();
                mEditPresetValue.setText("");
                mEditPresetDesc.setText("");
                break;
            case R.id.queryPreset:
                queryPreset(mCameraCode);
                break;

            default:
                break;
        }
    }

    //单击列表项时，云台跳转到预置位
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        int presetValue = mPresetInfos.get(position).getPresetValue();
        Log.d(TAG, "presetValue : " + presetValue);
        usePreset(mCameraCode, presetValue);
    }

    //长按列表项时,对预置位进行删除
    @Override
    public boolean onItemLongClick(AdapterView<?> parent, View view, final int position, long id) {
        new AlertDialog.Builder(PtzActivity.this).setTitle("确定删除？")
                .setMessage("预置位值:" + mPresetInfos.get(position).getPresetValue() + "\n预置位描述:" + mPresetInfos.get(position).getPresetDesc())
                .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {

                    }
                })
                .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        delPreset(mCameraCode, mPresetInfos.get(position).getPresetValue());
                        mAdapter.notifyDataSetChanged();
                    }
                }).show();
        return true;
    }


    class MySurfaceCallback implements SurfaceHolder.Callback {

        @Override
        public void surfaceCreated(SurfaceHolder holder) {
            Log.i(TAG, "------ surfaceCreated -----");
        }

        @Override
        public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            Log.i(TAG, "------ surfaceChanged ------");
            if (null != mPlayer) {
                mPlayer.changeDisplaySize(width, height);
            }
        }

        @Override
        public void surfaceDestroyed(SurfaceHolder holder) {
            Log.i(TAG, "------ surfaceDestroyed ------");
        }
    }
}
