package com.uniview.airimos.android_ts_sdk_sample;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.uniview.airimos.IMOSPlayer;
import com.uniview.airimos.android_ts_sdk_sample.adapter.AlarmAdapter;
import com.uniview.airimos.listener.OnGetAlarmListener;
import com.uniview.airimos.listener.OnPushAlarmListener;
import com.uniview.airimos.listener.OnQueryResourceListener;
import com.uniview.airimos.listener.OnStartLiveListener;
import com.uniview.airimos.manager.InfoManager;
import com.uniview.airimos.manager.ServiceManager;
import com.uniview.airimos.obj.AlarmInfo;
import com.uniview.airimos.obj.QueryCondition;
import com.uniview.airimos.obj.ResourceInfo;
import com.uniview.airimos.parameter.PushAlarmParam;
import com.uniview.airimos.parameter.QueryResourceParam;
import com.uniview.airimos.parameter.StartLiveParam;
import com.uniview.airimos.parameter.StreamParam;
import com.uniview.airimos.protocol.CameraInfo;
import com.uniview.airimos.service.KeepaliveService;

import java.util.List;

/**
 * Created by l02465 on 2016/4/12.
 */
public class PushAlarmActivity extends Activity implements View.OnClickListener, KeepaliveService.OnKeepaliveListener {

    private static final String TAG = "PushAlarmActivity";
    private SurfaceView mSurfaceView;
    private EditText mEditCameraCode;
    private Button mBtnStartLive;
    private Button mBtnStopLive;
    private Button mBtnQueryCam;
    private Button mBtnPushAlarm;
    private Button mBtnGetAlarm;
    private ListView mAlarmList;

    private IMOSPlayer mPlayer;
    private String mCameraCode;
    private KeepaliveService mService;
    private Intent mServiceintent;
    private boolean mBind = false;
    private boolean mNeedLogout = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_alarm);

        mSurfaceView = (SurfaceView) findViewById(R.id.alarm_surface_view);
        mEditCameraCode = (EditText) findViewById(R.id.cameraCode_edit);
        mBtnQueryCam = (Button) findViewById(R.id.query_camera);
        mBtnStartLive = (Button) findViewById(R.id.alarm_startLive);
        mBtnStopLive = (Button) findViewById(R.id.alarm_stopLive);
        mBtnGetAlarm = (Button) findViewById(R.id.getAlarm);
        mBtnPushAlarm = (Button) findViewById(R.id.pushAlarm);
        mAlarmList = (ListView) findViewById(R.id.alarmList);

        mBtnStartLive.setEnabled(false);
        mBtnStopLive.setEnabled(false);
        mBtnGetAlarm.setEnabled(false);
        mBtnPushAlarm.setEnabled(false);

        mBtnQueryCam.setOnClickListener(this);
        mBtnStartLive.setOnClickListener(this);
        mBtnStopLive.setOnClickListener(this);
        mBtnGetAlarm.setOnClickListener(this);
        mBtnPushAlarm.setOnClickListener(this);

        mSurfaceView.getHolder().addCallback(new MySurfaceHolderCallback());
        //初始化一个Player对象
        mPlayer = new IMOSPlayer();
        mPlayer.setDisplay(mSurfaceView.getHolder());

    }

    @Override
    protected void onStart() {
        super.onStart();
        if (!mBind) {
            mServiceintent = new Intent(PushAlarmActivity.this, KeepaliveService.class);
            startService(mServiceintent);
            bindService(mServiceintent, mConnect, Context.BIND_AUTO_CREATE);
        }
    }

    private ServiceConnection mConnect = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            KeepaliveService.KeepaliveBinder binder = (KeepaliveService.KeepaliveBinder) service;
            mService = binder.getService();
            mService.start(PushAlarmActivity.this);
            mBind = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            mBind = false;
        }
    };

    @Override
    protected void onStop() {
        super.onStop();
        if (mBind) {
            unbindService(mConnect);
            mBind = false;
        }

        if (mNeedLogout) {
            stopService(mServiceintent);
        }

    }

    /**
     * 启动实况
     *
     * @param cameraCode 摄像机编码
     */
    public void startLive(final String cameraCode) {
        try {
            //启动实况的结果监听
            OnStartLiveListener listener = new OnStartLiveListener() {
                @Override
                public void onStartLiveResult(long errorCode, String errorDesc, String playSession) {
                    mPlayer.stopPlay();

                    CameraInfo cameraInfo = new CameraInfo();
                    cameraInfo.setCameraCode(cameraCode);
                    mPlayer.startPlay(InfoManager.getServer(), InfoManager.getMediaPort(), playSession, cameraInfo);
                }

            };

            //设置启动实况的参数
            StartLiveParam p = new StartLiveParam();
            p.setCameraCode(cameraCode);
            p.setUseSecondStream(true); //使用辅流
            p.setBitrate(32 * 8);   //64KB的码率
            p.setFramerate(12);     //25帧率
            p.setResolution(StreamParam.ResolutionType.CIF);     //CIF分辨率

            //启动实况
            ServiceManager.startLive(p, listener);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void stopLive() {
        //停止实况，第二个参数是null表示不接收结果
        ServiceManager.stopLive(mPlayer.getPlaySession(), null);

        //停止Player播放解码
        mPlayer.stopPlay();
    }

    public void queryCameraRes() {
        try {
            OnQueryResourceListener listener = new OnQueryResourceListener() {
                @Override
                public void onQueryResourceResult(long errorCode, String errorDesc, List<ResourceInfo> resList) {
                    if (null == resList) {
                        return;
                    }
                    StringBuffer stringBuffer = new StringBuffer();
                    int size = resList.size();
                    for (int i = 0; i < size; i++) {
                        stringBuffer.append(resList.get(i).getResCode() + ",");
                        stringBuffer.append(resList.get(i).getResName() + ",");
                        stringBuffer.append(resList.get(i).getResType() + ",");
                        stringBuffer.append(resList.get(i).getResSubType() + ",");
                        stringBuffer.append(resList.get(i).getIsOnline() + "\n");

                        //找到第一个在线的摄像机
                        if (resList.get(i).getResType() == ResourceInfo.ResType.CAMERA && resList.get(i).getIsOnline()) {
                            mEditCameraCode.setText(resList.get(i).getResCode());
                            mCameraCode = resList.get(i).getResCode().trim();
                            if (!"".equals(mEditCameraCode.getText().toString())) {
                                mBtnStartLive.setEnabled(true);
                                mBtnStopLive.setEnabled(true);
                                mBtnPushAlarm.setEnabled(true);
                                mBtnGetAlarm.setEnabled(true);
                            }
                        }
                    }
                    Log.d(TAG, stringBuffer.toString());
                }
            };


            //查询摄像机资源参数
            QueryResourceParam param = new QueryResourceParam("", "", new QueryCondition(0, 100, false));

            //调用查询资源接口
            ServiceManager.queryResource(param, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    public void pushAlarm(String cameraCode) {

        PushAlarmParam param = new PushAlarmParam();
        param.setCameraCode(cameraCode);
        param.setAlarmType(401);
        param.setAlarmDesc("推送告警测试");

        try {
            OnPushAlarmListener listener = new OnPushAlarmListener() {
                @Override
                public void onPushAlarmResult(long errorCode, String errorDesc) {
                    if (0 != errorCode) {
                        Toast.makeText(PushAlarmActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(PushAlarmActivity.this, "告警推送成功", Toast.LENGTH_SHORT).show();
                    }
                }
            };
            //推送告警接口
            ServiceManager.pushAlarmMessage(param, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // 查询告警为阻塞调用，有告警会立即返回。如果需要一直查询，可以放在线程中连续调用
    public void getAlarm(int timeout) {
        try {
            OnGetAlarmListener listener = new OnGetAlarmListener() {
                @Override
                public void onGetAlarmResult(long errorCode, String errorDesc, List<AlarmInfo> alarmList) {
                    if (null == alarmList) {
                        return;
                    }
                    if (errorCode != 0) {
                        Toast.makeText(PushAlarmActivity.this, errorDesc, Toast.LENGTH_SHORT).show();
                    } else {
                        int size = alarmList.size();
                        StringBuffer stringBuffer = new StringBuffer();
                        for (int i = 0; i < size; i++) {
                            AlarmInfo alarmInfo = alarmList.get(i);
                            stringBuffer.append(alarmInfo.getAlarmSrcCode() + ",");
                            stringBuffer.append(alarmInfo.getAlarmSrcName() + ",");
                            stringBuffer.append(alarmInfo.getAlarmType() + ",");
                            stringBuffer.append(alarmInfo.getAlarmDesc() + ",");
                            stringBuffer.append(alarmInfo.getAlarmTime() + "\n");
                        }

                        AlarmAdapter myAdapter = new AlarmAdapter(getApplication(), alarmList);
                        mAlarmList.setAdapter(myAdapter);

                        Log.i(TAG, "告警信息:" + stringBuffer.toString());
                    }
                }
            };

            //获取告警
            ServiceManager.getAlarmMessage(timeout, listener);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.query_camera:
                queryCameraRes();
                break;
            case R.id.alarm_startLive:
                startLive(mCameraCode);
                break;
            case R.id.alarm_stopLive:
                stopLive();
                break;
            case R.id.pushAlarm:
                pushAlarm(mCameraCode);
                break;
            case R.id.getAlarm:
                getAlarm(10000);
                break;
            default:
                break;
        }
    }

    @Override
    public void onKeepaliveFailed() {
        mNeedLogout = true;
        Intent intent = new Intent(PushAlarmActivity.this, LoginActivity.class);
        startActivity(intent);
        Toast.makeText(PushAlarmActivity.this, "保活失败，进行退出", Toast.LENGTH_SHORT).show();
        finish();
    }

    class MySurfaceHolderCallback implements SurfaceHolder.Callback {

        @Override
        public void surfaceCreated(SurfaceHolder holder) {
            Log.i(TAG, "------ surfaceCreated -----");
        }

        @Override
        public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            Log.i(TAG, "------ surfaceChanged ------");
            if (null != mPlayer) {
                mPlayer.changeDisplaySize(width, height);
            }
        }

        @Override
        public void surfaceDestroyed(SurfaceHolder holder) {
            Log.i(TAG, "------ surfaceDestroyed ------");
        }
    }
}
